# RicherSyntaxPlugin (I hope)  
#(C) Franchino Bagnoli (bagnoli@dma.unifi.it) 20020514



# =========================
package TWiki::Plugins::RicherSyntaxPlugin; 	

# =========================
use vars qw(
        $web $topic $user $installWeb $VERSION $debug
        $exampleCfgVar
    );

$VERSION = '1.000';

# =========================
sub initPlugin
{
    ( $topic, $web, $user, $installWeb ) = @_;

    # check for Plugins.pm versions
    if( $TWiki::Plugins::VERSION < 1 ) {
        &TWiki::Func::writeWarning( "Version mismatch between RicherSyntaxPlugin and Plugins.pm" );
        return 0;
    }

    # Get plugin preferences, the variable defined by:          * Set EXAMPLE = ...
    $exampleCfgVar = &TWiki::Prefs::getPreferencesValue( "RICHERSYNTAXPLUGIN_EXAMPLE" ) || "default";

    # Get plugin debug flag
    $debug = &TWiki::Func::getPreferencesFlag( "RICHERSYNTAXPLUGIN_DEBUG" );

    # Plugin correctly initialized
    &TWiki::Func::writeDebug( "- TWiki::Plugins::RicherSyntaxPlugin::initPlugin( $web.$topic ) is OK" ) if $debug;
    return 1;
}

sub handleRicherSyntax {
	
	&TWiki::Func::writeDebug( "- TWiki::Plugins::RicherSyntaxPlugin::handleRicherSyntax") if $debug;
	
		my @tmp;
		

		# avoid changing things inside verbatim (and pre?) environments
	
		$_[0] =~ s/<(verbatim|pre|head)>(.*?)<\/\1>/"<$1>".push(@tmp, $2)."<\/$1>"/igse;


		# titles

		# header      
		# ======              => <H1>
		
		$_[0] =~ s/(^([ \t]*)(\w.*?)([ \t\r]*\n)\2(=+))/abs(length($3)-length($5))<=1?"---\+ $3\r\n":$1/gem;

		# header      
		# ------              => <H2>

		$_[0] =~ s/(^([ \t]*)(\w.*?)([ \t\r]*\n)\2(-+))/abs(length($3)-length($5))<=1?"---\+\+ $3\r\n":$1/gem;

		# header      
		# ^^^^^^               => <H3>

		$_[0] =~ s/(^([ \t]*)(\w.*?)([ \t\r]*\n)\2(\^+))/abs(length($3)-length($5))<=1?"---\+\+\+ $3\r\n":$1/gem;


		# header      
		# ~~~~~~               => <H4>

		$_[0] =~ s/(^([ \t]*)(\w.*?)([ \t\r]*\n)\2(\~+))/abs(length($3)-length($5))<=1?"---\+\+\+\+ $3\r\n":$1/gem;

	
		# process old-style descriptions. They are broken by continuations
			
	  $_[0] =~ s/^(\t+)(\w\S*?):\s/"<dt> $2<\/dt><dd>". &TWiki::emitList( "dl", "dd", length $1 )/gme;

		# Breaks. example: 

		# <space>* continuing
		# <space><space>line
		# <space><space>[...]
		# <space><space>after a break

		$_[0] =~ s/^([ \t]+)\.*\r*\n\1/$1$2%BR%%BR%/gm;
		
		# continuations:
		#
		# <space>bla bla
		# <space><space>bla

		while ($_[0] =~ s/^([ \t]*)([^ \t\r\n].+)\r*\n\1([ \t]+)([^\d\*:].*)([^\\]\r*\n)/$1$2$3$4$5/m){};


		# descriptions

		$_[0] =~ s/^(\t+):(.*?):\s(.*)$/<dl><dd><b>$2:<\/b>&nbsp;$3<\/dd><\/dl>/gm; 

		# then breaks
		
		$_[0] =~ s/%BR%/<br \/>/g;



		# putting back <verbatims>

		$_[0] =~ s/<(verbatim|pre|head)>(.*?)<\/\1>/"<$1>".$tmp[$2-1]."<\/$1>"/iges;
		


}



# =========================
sub commonTagsHandler
{
### my ( $text, $topic, $web ) = @_;   # do not uncomment, use $_[0], $_[1]... instead

    &TWiki::Func::writeDebug( "- RicherSyntaxPlugin::commonTagsHandler( $_[2].$_[1] )" ) if $debug;

    # This is the place to define customized tags and variables
    # Called by sub handleCommonTags, after %INCLUDE:"..."%

    # do custom extension rule, like for example:
    # $_[0] =~ s/%XYZ%/&handleXyz()/geo;
    # $_[0] =~ s/%XYZ{(.*?)}%/&handleXyz($1)/geo;
    # This handler is called by getRenderedVersion just before the line loop

    # do custom extension rule, like for example:
    # $_[0] =~ s/old/new/go;
		
		&handleRicherSyntax;
}

# =========================
sub startRenderingHandler
{
### my ( $text, $web ) = @_;   # do not uncomment, use $_[0], $_[1] instead

	

    &TWiki::Func::writeDebug( "- RicherSyntaxPlugin::startRenderingHandler( $_[1].$topic )" ) if $debug;

    # This handler is called by getRenderedVersion just before the line loop

    # do custom extension rule, like for example:
    # $_[0] =~ s/old/new/go;
		
	&handleRicherSyntax;


}	

# =========================
sub DISABLE_outsidePREHandler
{
### my ( $text ) = @_;   # do not uncomment, use $_[0] instead

#   &TWiki::Func::writeDebug( "- RicherSyntaxPlugin::outsidePREHandler( $web.$topic )" ) if $debug;

    # This handler is called by getRenderedVersion, in loop outside of <PRE> tag.
    # This is the place to define customized rendering rules.
    # Note: This is an expensive function to comment out.
    # Consider startRenderingHandler instead

}

# =========================
sub DISABLE_insidePREHandler
{
### my ( $text ) = @_;   # do not uncomment, use $_[0] instead

#   &TWiki::Func::writeDebug( "- RicherSyntaxPlugin::insidePREHandler( $web.$topic )" ) if $debug;

    # This handler is called by getRenderedVersion, in loop inside of <PRE> tag.
    # This is the place to define customized rendering rules.
    # Note: This is an expensive function to comment out.
    # Consider startRenderingHandler instead
}

# =========================
sub endRenderingHandler
{
### my ( $text ) = @_;   # do not uncomment, use $_[0] instead

    &TWiki::Func::writeDebug( "- RicherSyntaxPlugin::endRenderingHandler( $web.$topic )" ) if $debug;

    # This handler is called by getRenderedVersion just after the line loop


}

# =========================

1;
